#include "BFGS_Precond.h"
#include "QN_Precond.h"
#include "FiniteElemSpace.h"
#include "DOFAdmin.h"
#include "DOFVector.h"
#include "DOFMatrix.h"
#include "DOFIterator.h"

namespace AMDiS
{

  void BFGS_Precond::addPair(DOFVector<double> *s, DOFVector<double> *y,
			     double sy)
  {
    FUNCNAME("BFGS_Preconditioner::addPair()");

    if (!sy)
      sy = *s * *y;
    TEST_EXIT(sy > 0)("Vectors do not satisfy the curvature condition!\n");

    BFGS_Vectors newPair(s->getFESpace());
    newPair.setVectors(s, y, sy);

    if (k < m) {
      k++;
    } else {
      pairsVec.pop_back();
    }

    pairsVec.push_front(newPair);

    return;
  }

  void BFGS_Precond::precon(DOFVector<double> *vec)
  {
    FUNCNAME("BFGS_Preconditioner::precon()");

    int    i      = 0;
    double beta;
    double *alpha = GET_MEMORY(double, k);

    ::std::deque<BFGS_Vectors>::iterator pairsIterator(pairsVec.begin());

    while (pairsIterator != pairsVec.end()) {
      alpha[i]  = pairsIterator->s * *vec;
      alpha[i] /= pairsIterator->sy;
      aXpY(-alpha[i], pairsIterator->y, *vec, bound);
      ++i;
      ++pairsIterator;
    }

    diagPrecond(*matrix[row], vec, bound);

    while (pairsIterator != pairsVec.begin()) {
      --pairsIterator;
      --i;
      beta  = pairsIterator->y * *vec;
      beta /= pairsIterator->sy;
      aXpY(alpha[i]-beta, pairsIterator->s, *vec, bound);
    }
    
    FREE_MEMORY(alpha, double, k);
    
    return;
  }

}
